/*
 * ConnUnity.h - library for the connection to Unity
 *
 * based on code for a direct connection to Unity, and as such provided in the zip file
 * "Connecting Arduino to Unity.zip" by Tiuri de Jong
 * https://code.google.com/p/unity-arduino-serial-connection/
 *
 *
 * Adapted by Gidi van Liempd (gidi@geedesign.com) Jan 2013
*/

#ifndef ConnUnity_h
#define ConnUnity_h

#include "Boards.h"  /* Hardware Abstraction Layer */

class ConnUnity
{
  public:
  	ConnUnity(boolean bFakeDigital, boolean bFakeAnalog, char pinConfig[]);	// Constructor
  	void readVirtualDigital(byte pin, boolean value);	// Function to read a (virtual) digital pin value
  	void readVirtualAnalog(byte pin, int value);		// Function to read a (virtual) analog pin value
  	void processSerial();								// Function to process serial input and output

  private:
	// variables for connection Unity/Flash
	char serInString[ARRAY_LENGTH];  // the received string with output information max 40 characters
	char tempString[4];    // temp string for reading the pwm & output values
	int tempVal;           // temp int for reading pwm values
	int val;               // temp int for reading analog en digital inputs
	int stringLength;  	// expected length of string received from Flash/Unity
	int serInStringLength; // length of string received by Flash/Unity.
	int digOut;        // Number of digital outputs
	int digIn;         // Number of digital inputs
	boolean _bFakeDigital;	// TRUE if we use only fake digital inputs
	boolean _bFakeAnalog;	// TRUE if we use only fake analog inputs

	int digInputs[MAX_DIGITAL];     	// array that stores which pins are used as digital inputs max 12.
	int digInLastValues[MAX_DIGITAL];	// array that stores the last values read as digital inputs (or
										// fake inputs, derived from other computations)
	int analogInLastValues[MAX_ANALOG];	// array that stores the last values read as analog inputs (or
										// fake inputs, derived from other computations)
	int digOutputs[MAX_DIGITAL];    	// array that stores which pins are used as digital outputs max 12.
	int digOutLastValues[MAX_DIGITAL]; 	// array that stores the last values, an output only updates when there is a change

	char _pinConfig[MAX_DIGITAL];		// contains the actual (virtual) digital pin config used.Must be the same as in Unity!

  	/* private methods ------------------------------ */
  	void setupPins();							// Function to setup the pin configuration
  	void readSerialString (char *strArray);		// Function to read a string from the serialport and store it in an array
  	boolean isStringEmpty(char *strArray);		// Function to find out wether the array is empty or not
  	void sendInputsToFlash(boolean error); 		// Function which sends Arduino input values to Flash/Unity
  	void updateArduinoOutputs();				// Function to update the Arduino outputs, with values received from Flash/Unity
  	void eraseStrings();						// Function to erase contents in serInString array
};

#endif /* ConnUnity_h */
